<?php 

namespace Management\Services;

use Doctrine\ORM\EntityManager;
use Management\Util;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class DashboardDataService {

	const NOME_TODAS_UNIDADES = 'Todas Unidades';
	const MAX_DIAS_ANTERIORES = 20;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

	public function getQuantidadeVendasConcluidas ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_QUANTIDADE_VENDAS_CONCLUIDAS, $params);
	}
	
	public function getQuantidadeVendasCanceladas ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_QUANTIDADE_VENDAS_CANCELADAS, $params);
	}
	
	public function getQuantidadeVendasConcluidasByFilial ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_QUANTIDADE_VENDAS_CONCLUIDAS_FILIAL, $params);
	}
	
	public function getQuantidadeVendasCanceladasByFilial ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_QUANTIDADE_VENDAS_CANCELADAS_FILIAL, $params);
	}
	
	public function getTotalVendaFilial ($params) {
		$resposta = new ObjetoResposta(200, null);
		
		if($params[':MENOSDIAS'] < 0 ||  $params[':DATA'] != date('d/m/y')) {
			$resposta->setStatus(300);
			$params[':HORA'] = 23;
		} else
	    	$params[':HORA'] = date('G');
		$retornoQuery = $this->entityManager->getConnection()->fetchAll(self::GET_VENDA_POR_FILIAL, $params);
		
		$consultas = 1;
		while (empty($retornoQuery) && $consultas < self::MAX_DIAS_ANTERIORES) {
		    $params[':HORA'] = 23;
			// Vendas de dias retroativos
			$resposta->setStatus(300);
			$subtract = $consultas;
			
			$dataAtual = $params[':DATA'];
			list($dd,$mm,$yy) = explode('/',$dataAtual);
			$date = sprintf('20%s-%s-%s',$yy,$mm,$dd);

			$newdate = strtotime ( '-1 day' , strtotime($date)) ;
			$params[':DATA'] = date ('d/m/y', $newdate );
			$retornoQuery = $this->entityManager->getConnection()->fetchAll(self::GET_VENDA_POR_FILIAL, $params);
			
			if(!empty($retornoQuery))
				break;
			else
				$consultas++;
		}
		
		if($consultas < self::MAX_DIAS_ANTERIORES) {
    		$i = 0;
    		foreach($retornoQuery as $qry) {
    			
    			$totalVendido = $qry['TOTALVEND'];
    			$meta = $qry['META'];
    			
    			if($meta > 0) 
    				$valorMeta = $meta - $totalVendido;
    			else
    				$valorMeta = 0;
    
    			if ($valorMeta < 0)
    				$valorMeta = 0;
    				
    			$qry['DADOSMETA'] = array (
    				array(
    					'LABELFILIAL' => "Total Vendido",
    					'TOTALVEND'   => $totalVendido,
    					'META'        => $meta,
    					'ORDER'       => "0"),
    				array(
    					'LABELFILIAL' => "Diferença Meta Acumulada",
    					'TOTALVEND'   => $valorMeta,
    					'ORDER'       => "1")
    			);
    			$retornoQuery[$i]['DADOSMETA'] = $qry['DADOSMETA'];
    			$i++;
    			
    		}
			$resposta->setData($params[':DATA']);
			$resposta->setDados($retornoQuery);
			
		
		} else {
			// Não foi encontrada nenhuma venda
			$resposta->setStatus(100);
		}
		
		
		
		return $resposta;
		
	}
	
	public function getTotalVenda ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_VENDA_TOTAL, $params);
	}
	
	public function getTicketMedio ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_TICKET_MEDIO_GERAL, $params);
	}
	
	public function getTicketMedioFilial ($params) {
		return $this->entityManager->getConnection()->fetchAll(self::GET_TICKET_MEDIO_FILIAL, $params);
	}
	
	public function getVendaModalidade ($params) {
		$retorno = array();
		
		$modalidades = ["Mesa", "Balcão", "Delivery", "TAA", "Comanda", "Outros"];
		$resultado = $this->entityManager->getConnection()->fetchAll(self::GET_VENDA_MODALIDADE, $params);
		
		return $resultado;
	}
	
	public function getVendaModalidadeFilial ($params) {
		$resultado = $this->entityManager->getConnection()->fetchAll(self::GET_VENDA_MODALIDADE_FILIAL, $params);
		if(empty($resultado))
			$resultado = array(
				array(
					"UNIDADE" => "Todas Unidades",
					"MES"     => 0,
					"BAL"	  => 0,
					"TAA"     => 0,
					"CMD"     => 0,
					"OUTROS"  => 0)
				);
		return $resultado;
	}
	
	public function getVendaHora ($params) {

		$retornoArray = array();
		
		$unidades = array();

        	
		$data = $params[':DATA'];
		
		$horaAtual = 23;
	
		for ($i = 0; $i <= $horaAtual+1; $i++) {
			
			$params[':HORA'] =  $i;
			$query = $this->entityManager->getConnection()->fetchAll(self::GET_VENDA_POR_HORA, $params);
			
			$j = 0;
			$total = 0;
			
			foreach($query as $resultado) {
				
				$arrayVendas = array();
				
				$total = $query[$j]['TOTALVEND'];
				$data = $query[$j]['DATAVENDA'];
				$cdfilial = $query[$j]['CDFILIAL'];
				$nmfilial = $query[$j]['NMFILIAL'];
				$meta = $query[$j]['META'];
					
				$unidade = $this->getObjeto($unidades, $cdfilial, $nmfilial);
				$unidade->addVendas($i.":00", $total, $meta);
	            $j++;
			    
			}
        }
        
        if($data != date('d/m/y')) {
			$horaAtual = "23:00";
		} else
	    	$horaAtual = date('G').":00";
	    	
        foreach($unidades as $unidade) {
        	$retornoArray[] = array(
        		"LABELFILIAL"     => $unidade->getLabelFilial(),
        		"VENDAS"		  => $unidade->getVendas($horaAtual)
        	);
        }
		return $retornoArray;
	}
	
	public function getObjeto (array & $array, $cdfilial, $nmfilial) {
		
		$objUnidade;
		$i = 0;
		
		foreach($array as $unidade) {
			if($unidade->getCdFilial() == $cdfilial) {
				$objUnidade = $unidade;
				break;
			}
		}
		
		if(empty($objUnidade)) {
			$objUnidade = new ObjetoUnidade($cdfilial, $nmfilial);
			$array[] = $objUnidade;
		}
		
		return $objUnidade;
		
	}
	
	public function syncDashboardData () {
		$this->entityManager->getConnection()->executeQuery(self::UPDATE_VENDAS_DIA);
		$this->entityManager->getConnection()->executeQuery(self::LIMPAR_TABELA_VENDADIAMODALIDADE);
		$this->entityManager->getConnection()->executeQuery(self::LIMPAR_TABELA_VENDADIATOTAL);
		$this->entityManager->getConnection()->executeQuery(self::LIMPAR_TABELA_VENDAHORATOTAL);
		$procedure = new StoredProcedure($this->entityManager->getConnection(), 'REPLICA_VENDAS_RM');
        $retorno = $procedure->execute();
        return $retorno;
	}
	

	
	const GET_QUANTIDADE_VENDAS_CONCLUIDAS = "
	    SELECT NVL(SUM(NVL(NRVENDACON, 0)),0) AS VENDAS
		FROM VENDADIATOTAL
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND CDFILIAL IN
		  (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
		AND NRORG = :NRORG
    ";
    
    const GET_QUANTIDADE_VENDAS_CANCELADAS = "
		SELECT NVL(SUM(NVL(NRVENDACAN, 0)), 0) AS VENDAS
		FROM VENDADIATOTAL
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND CDFILIAL IN
		  (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
		AND NRORG = :NRORG	
    ";
    
    const GET_QUANTIDADE_VENDAS_CONCLUIDAS_FILIAL = "
	    SELECT NVL(SUM(NVL(NRVENDACON, 0)), 0) AS VENDAS
		FROM VENDADIATOTAL
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND CDFILIAL IN
		  (SELECT CDFILIAL 
		     FROM FILIOPER 
		    WHERE CDOPERADOR = :CDOPERADOR
		      AND CDFILIAL   = :CDFILIAL)
        	  AND NRORG = :NRORG
    ";
    
    const GET_QUANTIDADE_VENDAS_CANCELADAS_FILIAL = "
		SELECT NVL(SUM(NVL(NRVENDACAN, 0)), 0) AS VENDAS
		FROM VENDADIATOTAL
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND CDFILIAL IN
		  (SELECT CDFILIAL 
		     FROM FILIOPER 
		    WHERE CDOPERADOR = :CDOPERADOR
		      AND CDFILIAL   = :CDFILIAL)
		AND NRORG = :NRORG	
    ";
    
    
    const GET_VENDA_POR_FILIAL = "
    	SELECT * 
	    FROM (SELECT F.CDFILIAL || ' - ' || F.NMFILIAL AS LABELFILIAL,
					 F.CDFILIAL AS CDFILIAL, NVL(V.VRTOTAL, 0) AS TOTALVEND, NVL(VRMETAHORAACUM, 0) AS META
				FROM VENDADIATOTAL V
			    JOIN FILIAL F 
			      ON V.CDFILIAL   = F.CDFILIAL
	    		LEFT JOIN METAHORAREAL MHR 
	    		  ON V.CDFILIAL = MHR.CDFILIAL
	    		 AND MHR.HORA = TO_NUMBER(:HORA)
				 AND TRUNC(MHR.DATA) = TO_DATE(:DATA, 'dd/mm/yy')
			   WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
				 AND V.CDFILIAL IN (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
			AND V.NRORG = :NRORG
			ORDER BY 3 DESC
			)
		WHERE ROWNUM <= 30
    ";
    
    const GET_VENDA_TOTAL = "
	    SELECT    '".self::NOME_TODAS_UNIDADES."' AS LABELFILIAL,
          NVL(SUM(NVL(V.VRTOTAL, 0)), 0) AS TOTALVEND,
		  0.0                  AS META
		FROM VENDADIATOTAL V
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND V.CDFILIAL IN
		  (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
		AND V.NRORG = :NRORG
    ";
    
    const GET_TICKET_MEDIO_GERAL = "
	    SELECT    '".self::NOME_TODAS_UNIDADES."' AS LABELFILIAL,
          NVL(AVG(NVL(V.VRTICKETMED, 0)), 0) AS TICKETMEDIO,
		  0.0                  AS META
		FROM VENDADIATOTAL V
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND V.CDFILIAL IN
		  (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
		AND V.NRORG = :NRORG
    ";
    
	const GET_TICKET_MEDIO_FILIAL = "
		SELECT * 
		  FROM (
				 SELECT F.CDFILIAL || ' - ' || F.NMFILIAL AS LABELFILIAL, NVL(V.VRTICKETMED, 0) AS TICKETMEDIO, NVL(MDR.VRTCKMED,0)   AS META
				   FROM VENDADIATOTAL V
				   JOIN FILIAL F
					 ON V.CDFILIAL   = F.CDFILIAL
				   LEFT JOIN METADIAREAL MDR
					 ON F.CDFILIAL = MDR.CDFILIAL
					AND TRUNC(MDR.DATA) = TO_DATE(:DATA, 'dd/mm/yy')
				  WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
					AND V.CDFILIAL IN (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
					AND V.NRORG = :NRORG
				  ORDER BY 2 DESC
				 )
		WHERE ROWNUM <= 30
	";
	
	const GET_VENDA_MODALIDADE = "
		SELECT MODALIDADE, NVL(SUM(VENDAS), 0) AS TOTALVEND, '".self::NOME_TODAS_UNIDADES."' AS LABELFILIAL FROM (
		SELECT
		  CASE SUBSTR(IDMODALIDADE, 0, 3)
		    WHEN 'DLV'
		    THEN 'Delivery'
		    WHEN 'MES'
		    THEN 'Mesa'
		    WHEN 'BAL'
		    THEN 'Balcão'
		    WHEN 'TAA'
		    THEN 'TAA'
		    WHEN 'CMD'
		    THEN 'Comanda'
		    ELSE 'Outros'
		  END AS MODALIDADE,
		  NVL(VRTOTAL, 0) AS VENDAS
		FROM VENDADIAMODALIDADE
		WHERE TRUNC(DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		AND CDFILIAL IN
		  (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
		AND INSTR('MES_BAL_DLV_TAA_CMD_VND', SUBSTR(IDMODALIDADE, 0, 3) ) > 0
		AND NRORG                                           = :NRORG) X
		GROUP BY MODALIDADE
	";
	
	const GET_VENDA_MODALIDADE_FILIAL = "
	SELECT * FROM (
	SELECT X.CDFILIAL,
	  X.LABELFILIAL,
	  CASE X.MODALIDADE
	      WHEN 'DLV'
	      THEN 'Delivery'
	      WHEN 'MES'
	      THEN 'Mesa'
	      WHEN 'BAL'
	      THEN 'Balcão'
	      WHEN 'TAA'
	      THEN 'TAA'
	      WHEN 'CMD'
	      THEN 'Comanda'
	      ELSE 'Outros'
	  END             AS MODALIDADE,
	  X.VENDAS AS TOTALVEND,
	  NVL(MM.VRMETADIA, 0) AS META,
    NVL(MMD.VRMETAMODALIDADE, 0) AS METAMODALIDADE
	  FROM (SELECT
            VDM.CDFILIAL,
            F.CDFILIAL || ' - ' || F.NMFILIAL AS LABELFILIAL,
            SUBSTR(VDM.IDMODALIDADE, 0, 3) AS MODALIDADE,
            NVL(VRTOTAL, 0) AS VENDAS    
          FROM VENDADIAMODALIDADE VDM
          JOIN FILIAL F
          ON F.CDFILIAL = VDM.CDFILIAL
          WHERE TRUNC(VDM.DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
          AND VDM.CDFILIAL IN
            (SELECT CDFILIAL FROM FILIOPER WHERE CDOPERADOR = :CDOPERADOR)
          AND INSTR('MES_BAL_DLV_TAA_CMD_VND_OUT', SUBSTR(VDM.IDMODALIDADE, 0, 3)) > 0
          AND VDM.NRORG = :NRORG
          ORDER BY CDFILIAL) X
	  LEFT JOIN METADIAREAL MM
		ON X.CDFILIAL = MM.CDFILIAL
		AND TRUNC(MM.DATA) = TO_DATE(:DATA, 'dd/mm/yy')
    LEFT JOIN METAMODALIDADEREAL MMD
		ON X.CDFILIAL = MMD.CDFILIAL
    AND MMD.IDMODALIDADE = X.MODALIDADE
		AND TRUNC(MMD.DATA) = TO_DATE(:DATA, 'dd/mm/yy')
	  ORDER BY 4 DESC
	  )
	  WHERE ROWNUM <= 30
	";
	
	const GET_VENDA_POR_HORA = "
	SELECT * FROM (
		SELECT X.CDFILIAL        AS CDFILIAL,
		  X.NMFILIAL             AS NMFILIAL,
		  X.DATAVENDA            AS DATAVENDA,
    	  X.HORA                 AS HORA,
		  NVL(SUM(X.VRTOTAL), 0) AS TOTALVEND,
		  X.META                 AS META
		FROM
		  (SELECT F.CDFILIAL,
		    NVL(VH.VRTOTALACUMULADO, 0) AS VRTOTAL,
		    NVL(TO_CHAR(VH.DTVENDA, 'dd/mm/yy'), :DATA) DATAVENDA,
		    NVL(VRMETAHORAACUM, 0) AS META,
            NVL(MHR.HORA, :HORA) AS HORA,
		    FILI.NMFILIAL
		  FROM FILIOPER F
		  LEFT JOIN VENDAHORATOTAL VH
		  ON F.CDFILIAL = VH.CDFILIAL
		  AND TRUNC(VH.DTVENDA) = TO_DATE(:DATA, 'dd/mm/yy')
		  AND VH.HORAVENDA = :HORA
		  LEFT JOIN METAHORAREAL MHR
		  ON F.CDFILIAL = MHR.CDFILIAL
		  AND MHR.HORA = :HORA
          AND TRUNC(MHR.DATA) = TO_DATE(:DATA, 'dd/mm/yy')
		  JOIN FILIAL FILI
		  ON F.CDFILIAL   = FILI.CDFILIAL
		  WHERE F.CDOPERADOR                       = :CDOPERADOR
		  AND F.NRORG                              = :NRORG) X
		WHERE X.VRTOTAL > 0  
		GROUP BY X.CDFILIAL, X.DATAVENDA, X.HORA, X.NMFILIAL, X.DATAVENDA, X.META
		ORDER BY 5 DESC
	  )
	  WHERE ROWNUM <= 30
	";  
	
	const UPDATE_VENDAS_DIA = "
		UPDATE VENDA
			SET IDCONTVEND = 'N'
		WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const LIMPAR_TABELA_VENDADIAMODALIDADE = "
		DELETE FROM VENDADIAMODALIDADE
        WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const LIMPAR_TABELA_VENDADIATOTAL = "
		DELETE FROM VENDADIATOTAL
        WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";
	
	const LIMPAR_TABELA_VENDAHORATOTAL = "
		DELETE FROM VENDAHORATOTAL
        WHERE TO_DATE(TO_CHAR(DTVENDA, 'DD/MM/YYYY'))  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
	";

} 